#import "Xeverywhere.h"

int max_depth;
int speed;


@implementation Xeverywhere	

- (id) initWithFrame:(NSRect)frameRect isPreview:(BOOL) preview
{
    NSString *identifier = [[NSBundle bundleForClass:[self class]] bundleIdentifier];
    ScreenSaverDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:identifier];
    id tObject;
    
    preview_=preview;
    
    isConfiguring_=NO;
    
    if (preview_==YES)
    {
        mainScreen_=YES;
    }
    else
    {
        mainScreen_= (frameRect.origin.x==0 && frameRect.origin.y==0) ? YES : NO;
    }
    
    mainScreenOnly_=[defaults integerForKey:@"MainScreen Only"];
    
    self = [super initWithFrame:frameRect isPreview:preview];
    
    if (self)
    {
        
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            NSOpenGLPixelFormatAttribute attribs[] = 
            {
                NSOpenGLPFADoubleBuffer,
                NSOpenGLPFADepthSize,(NSOpenGLPixelFormatAttribute)16,
                NSOpenGLPFAMinimumPolicy,
                (NSOpenGLPixelFormatAttribute)0
            };
        
            NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
            
            if (format!=nil)
            {
                _view = [[[NSOpenGLView alloc] initWithFrame:NSZeroRect pixelFormat:format] autorelease];
                [self addSubview:_view];
            
                speed=200;
                
                timer_=NULL;
                
                #if DEBUG
                frameCount=0;
                #endif

                
                tObject=[defaults objectForKey:@"Subdivision"];
            
                if (tObject==nil)
                {
                    max_depth=3;
                }
                else
                {
                    [self readDefaults:defaults];
                }
                
                [self setAnimationTimeInterval:0.04];
            }
        }
    }
    
    return self;
}

- (void) setFrameSize:(NSSize)newSize
{
	[super setFrameSize:newSize];
    
	if (_view!=nil)
    {
        [_view setFrameSize:newSize];
    }
}

- (void) drawRect:(NSRect) inFrame
{
	[[NSColor blackColor] set];
            
    NSRectFill(inFrame);
    
    if (_view==nil)
    {    
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            NSRect tFrame=[self frame];
            NSRect tStringFrame;
            NSDictionary * tAttributes;
            NSString * tString;
            NSMutableParagraphStyle * tParagraphStyle;
            
            tParagraphStyle=[[NSParagraphStyle defaultParagraphStyle] mutableCopy];
            [tParagraphStyle setAlignment:NSCenterTextAlignment];
            
            tAttributes = [NSDictionary dictionaryWithObjectsAndKeys:[NSFont systemFontOfSize:[NSFont systemFontSize]],NSFontAttributeName,[NSColor whiteColor],NSForegroundColorAttributeName,tParagraphStyle,NSParagraphStyleAttributeName,nil];
            
            [tParagraphStyle release];
            
            tString=NSLocalizedStringFromTableInBundle(@"Minimum OpenGL requirements\rfor this Screen Effect\rnot available\ron your graphic card.",@"Localizable",[NSBundle bundleForClass:[self class]],@"No comment");
            
            tStringFrame.origin=NSZeroPoint;
            tStringFrame.size=[tString sizeWithAttributes:tAttributes];
            
            tStringFrame=SSCenteredRectInRect(tStringFrame,tFrame);
            
            [tString drawInRect:tStringFrame withAttributes:tAttributes];
            
            return;
        }
    }
}

- (void) animateOneFrame
{
	if (isConfiguring_==NO && _view!=nil)
    {
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            [[_view openGLContext] makeCurrentContext];
            
            draw_gasket(&gs);
            
            [[_view openGLContext] flushBuffer];
        }
    }
}


- (BOOL) hasConfigureSheet
{
	return (_view!=nil);
}

- (void) startAnimation
{
	#if DEBUG
	timer_ = [[NSTimer scheduledTimerWithTimeInterval:10
                                                   target:self
                                                 selector:@selector(frameLog:)
                                                 userInfo:nil
                                                  repeats:YES] retain];
	#endif
    
	[super startAnimation];
    
    if (isConfiguring_==NO && _view!=nil)
    {
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            NSSize tSize;
            const GLint i=1;
			
            [self lockFocus];
            
			[[_view openGLContext] makeCurrentContext];
            
            glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
            glClear(GL_COLOR_BUFFER_BIT);
            
            [[_view openGLContext] flushBuffer];
            
            tSize=[_view frame].size;
            
            gs.view_rotx = 0;
            gs.view_roty = 0;
            gs.view_rotz = 0;
            gs.angle = 0;

            reshape((int) tSize.width, (int) tSize.height);

            pinit(&gs);
			
			CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval,&i);
            
            [self unlockFocus];
        }
    }
    
    [super startAnimation];
}

- (void)stopAnimation
{
	[super stopAnimation];
    
    if (timer_!=NULL)
	{
            [timer_ invalidate];
            [timer_ release];
            timer_=NULL;
	}
    
	if (_view!=nil)
    {
        if (mainScreenOnly_!=NSOnState || mainScreen_==YES)
        {
            [[_view openGLContext] makeCurrentContext];
            
            if (glIsList(gs.gasket1)) glDeleteLists(gs.gasket1, 1);
        }
    }
}

- (void) frameLog:(id) inSender
{
    fprintf(stderr,"%f frames/second\n",frameCount/10.0);
	frameCount=0;
}

- (void) readDefaults:(ScreenSaverDefaults *) inDefaults
{
    max_depth=[inDefaults integerForKey:@"Subdivision"];
    
    mainScreenOnly_=[inDefaults integerForKey:@"MainScreen Only"];
}

- (void) writeDefaults
{
    NSString *identifier = [[NSBundle bundleForClass:[self class]] bundleIdentifier];
    ScreenSaverDefaults *inDefaults = [ScreenSaverDefaults defaultsForModuleWithName:identifier];
    
	mainScreenOnly_=[IBmainScreen_ state];
    
    [inDefaults setInteger:max_depth forKey:@"Subdivision"];
    
    [inDefaults setInteger:mainScreenOnly_ forKey:@"MainScreen Only"];
    
    [inDefaults  synchronize];
}

- (void) setDialogValue
{
    [IBsubdivisionSlider_ setIntValue:max_depth];
    
    [IBmainScreen_ setState:mainScreenOnly_];
}

- (NSWindow*)configureSheet
{
    isConfiguring_=YES;
    
    if (IBconfigureSheet_ == nil)
    {
        [NSBundle loadNibNamed:@"ConfigureSheet" owner:self];
        
        [IBversion_ setStringValue:[[[NSBundle bundleForClass:[self class]] infoDictionary] objectForKey:@"CFBundleVersion"]];
    }
    
    [self setDialogValue];
    
    [self setSubdivision:IBsubdivisionSlider_];
    
    return IBconfigureSheet_;
}

- (IBAction)closeSheet:(id)sender
{
    if ([sender tag]==NSOKButton)
    {
        [self writeDefaults];
    }
    else
    {
        NSString *identifier = [[NSBundle bundleForClass:[self class]] bundleIdentifier];
        ScreenSaverDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:identifier];
        id tObject;
        
        tObject=[defaults objectForKey:@"Subdivision"];
            
        if (tObject==nil)
        {
            max_depth=3;
        }
        else
        {
            [self readDefaults:defaults];
        }
    }
    
    isConfiguring_=NO;
    
    if ([self isAnimating]==YES)
    {
        [self stopAnimation];
        [self startAnimation];
    }
    
    [NSApp endSheet:IBconfigureSheet_];
}


- (IBAction)setSubdivision:(id)sender
{
    unsigned long tFaces;
    
    max_depth=[sender intValue];
    
    tFaces=6*pow(9,max_depth);
    
    [IBfaces_ setStringValue:[NSString stringWithFormat:NSLocalizedStringFromTableInBundle(@"%d faces",@"Localizable",[NSBundle bundleForClass:[self class]],@"No comment"),tFaces]];
}


@end
